'use client'

import React, { useState, useEffect } from 'react'
import { Project, Tag, projectsSchema } from '@/typings/project'
import { ProjectCard } from '@/components/ProjectCard'
import projectsData from '../../public/projects.json'

const Page = () => {
  const parsedProjects = projectsSchema.safeParse(projectsData)

  if (!parsedProjects.success) {
    console.error('Invalid project data', parsedProjects.error)
    return <div>Error loading projects</div>
  }

  const projects: Project[] = parsedProjects.data

  const [currentPage, setCurrentPage] = useState(1)
  const [itemsPerPage, setItemsPerPage] = useState<number>(4)
  const [filterTag, setFilterTag] = useState<Tag | null>(null)
  const [sortOrder, setSortOrder] = useState<'asc' | 'desc'>('asc')

  const handleFilterChange = (event: React.ChangeEvent<HTMLSelectElement>) => {
    setFilterTag(
      event.target.value === 'All' ? null : (event.target.value as Tag)
    )
    setCurrentPage(1) // Reset to the first page when filter changes
  }

  const handleSortChange = (event: React.ChangeEvent<HTMLSelectElement>) => {
    setSortOrder(event.target.value as 'asc' | 'desc')
    setCurrentPage(1) // Reset to the first page when sort order changes
  }

  useEffect(() => {
    const handleResize = () => {
      const cardWidth = 240 // Width of a single card (adjust as needed)
      const windowWidth = window.innerWidth
      const newItemsPerPage = Math.floor(windowWidth / cardWidth)
      setItemsPerPage(newItemsPerPage > 0 ? newItemsPerPage : 1)
    }

    handleResize()

    window.addEventListener('resize', handleResize)

    return () => window.removeEventListener('resize', handleResize)
  }, [])

  const filteredProjects = filterTag
    ? projects.filter(project => project.tags.includes(filterTag))
    : projects

  const sortedProjects = [...filteredProjects].sort((a, b) => {
    if (sortOrder === 'asc') {
      return a.name.localeCompare(b.name)
    } else {
      return b.name.localeCompare(a.name)
    }
  })

  const indexOfLastItem = currentPage * itemsPerPage
  const indexOfFirstItem = indexOfLastItem - itemsPerPage
  const currentProjects = sortedProjects.slice(
    indexOfFirstItem,
    indexOfLastItem
  )

  const totalPages = Math.ceil(sortedProjects.length / itemsPerPage)

  return (
    <div className="flex min-h-screen flex-col items-center p-4 md:p-8">
      <div className="mb-8 text-2xl font-bold">Matz Hilven</div>
      <div className="mb-4 flex gap-4">
        <select onChange={handleFilterChange} className="rounded border p-2">
          <option value="All">All Tags</option>
          {[...new Set(projects.flatMap(project => project.tags))].map(tag => (
            <option key={tag} value={tag}>
              {tag}
            </option>
          ))}
        </select>
        <select onChange={handleSortChange} className="rounded border p-2">
          <option value="asc">Sort by Name (A-Z)</option>
          <option value="desc">Sort by Name (Z-A)</option>
        </select>
      </div>
      <div className="mx-4 flex flex-wrap justify-center gap-4">
        {currentProjects.map((project: Project, index: number) => (
          <ProjectCard project={project} key={index} />
        ))}
      </div>
      <div className="mt-4 flex gap-4">
        <button
          onClick={() => setCurrentPage(currentPage - 1)}
          disabled={currentPage === 1}
          className="rounded border px-4 py-2 disabled:opacity-50"
        >
          Previous
        </button>
        <button
          onClick={() => setCurrentPage(currentPage + 1)}
          disabled={currentPage === totalPages}
          className="rounded border px-4 py-2 disabled:opacity-50"
        >
          Next
        </button>
      </div>
    </div>
  )
}

export default Page
